extern crate regex;
extern crate structopt;

mod config;
mod dirs;
mod error;
mod files;
mod skel;

use std::path::PathBuf;

use structopt::StructOpt;

use config::Config;

fn main() -> error::MkrootResult<()> {
	let config: Config = StructOpt::from_args();

	if config.verbose {
		println!(
			"Building root fs for {:?} in {}",
			&config.files,
			&config.root_dir.display()
		);
	}

	dirs::check(&config)?;
	let files = files::Files::gather(&config)?;
	dirs::create(&config)?;
	files.copy(&config)?;

	let mut d = PathBuf::from(&config.root_dir);
	d.push("lib");
	files::Files::set_linker_permissions(&files.libs, &d, config.verbose)?;

	let mut d = PathBuf::from(&config.root_dir);
	d.push("lib64");
	files::Files::set_linker_permissions(&files.lib64s, &d, config.verbose)?;

	if let Err(e) = files::os_release(&config) {
		return Err(error::MkrootError::from(format!(
			"Error creating etc/os-release: {}",
			e
		)));
	}

	if config.skel != PathBuf::new() {
		skel::copy(&config)?;
	}

	Ok(())
}
